(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gtwinnt.dgnodes";*)
(*require "$.gtwinnt.dgpcommon";*)
(*require "$.mlabstractuiwinnt.pnotifiers";*)
(*require "protocolaccessorcontr";*)
(*require "$.errorhandling.exceptions";*)
(*require "simplesmlprettyprint";*)
(*require "$.mlabstractuiwinnt.uiwindowstoportable";*)
(*require "simplesmluieval";*)
(*require "simplesml97parser";*)

signature OPERATOR_CONTR =
  sig
    val createController : DGNodes.t ->
                           {loadFileName : PString.t,
                            saveFileName : PString.t,
                            onLoad : PNotify.t,
                            onSave : PNotify.t,
                            messageString : PString.t} ->
                           ProtocolAccessorContr.t ->
                           {isTCConstructed : DGNodes.t -> bool,
                            createNamedControl : string -> DGNodes.t} ->
                           unit
  end

functor FOperatorContr() :> OPERATOR_CONTR =
  struct
    fun createController masterNode
                         {loadFileName, saveFileName,
                          onLoad, onSave,
                          messageString}
                         {access, clear,
                          getAccessed, checkBinding,
                          addBindDesc}
                         {isTCConstructed,
                          createNamedControl} =
          let
            fun outF str =
                  PString.write messageString str
          
            fun doLoad _ () =
                  let
                    val absSyn =
                          Excn.checkedExecOutF
                            outF
                            SimpleSML97Parser.parseFile
                            (PString.read loadFileName)
                            nil
                  in
                    Excn.checkedExecOutF
                      outF
                      (SimpleSMLUIEval.smlAbsSynToUI
                         masterNode addBindDesc createNamedControl)
                      absSyn
                      ()
                  end

            fun doSave _ () =
                  case getAccessed () of
                    NONE => outF "You must select a window to be saved!"
                  | SOME(aNode) =>
                      if DGNodes.typeName aNode = "Window" then
                        Excn.checkedExecOutF
                          outF
                          (fn () => (SimpleSMLPrettyPrint.saveFile
                             checkBinding isTCConstructed aNode
                             (PString.read saveFileName);
                             outF "Save was successful!"))
                          () ()
                      else
                        outF "You must select a window to be saved!"

            val _ = PNotify.addNotifier onLoad doLoad
            val _ = PNotify.addNotifier onSave doSave
          in
            ()
          end
  end

structure OperatorContr = FOperatorContr()
