(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "protocolaccessorcontr";*)
(*require "operatorcontr";*)
(*require "uicontrolpalettecontr";*)
(*require "operator";*)
(*require "protocolaccessor";*)
(*require "uicontrolpalette";*)
(*require "$.errorhandlingwinnt.exceptions";*)
(*require "$.mlabstractuiwinnt.uiwindowstoportable";*)

signature THE_CONSTRUCT =
  sig
    val start : unit -> unit
  end

functor FTheConstruct() :> THE_CONSTRUCT =
  struct
    fun start () =
          let
            fun app () =
                  let
                    val masterNode = uiCore.createMasterNode ()

                    val accessorGUI =
                          ProtocolAccessor.createGraph masterNode
                    val controlPaletteGUI =
                          UIControlPalette.createGraph masterNode
                          
                    val accessorContr =
                          ProtocolAccessorContr.createController
                            masterNode accessorGUI
                    val controlPaletteContr =
                          UIControlPaletteContr.createController
                              masterNode
                              accessorContr
                              controlPaletteGUI

                    val operatorGUI =
                          Operator.createGraph masterNode
                    val _ = OperatorContr.createController
                              masterNode operatorGUI
                              accessorContr controlPaletteContr

                    val _ = Excn.checkedExec
                              uiCore.createGUI masterNode ()
                  in
                    uiCore.runGUI masterNode
                  end
          in
            uiCore.runApp app
          end
  end

structure TheConstruct = FTheConstruct()
