(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.basis.__list";*)
(*require "$.gtwinnt.dgnodes";*)
(*require "$.gtwinnt.dgshorthand";*)
(*require "$.gtwinnt.dgpcommon";*)
(*require "$.mlabstractuiwinnt.winregistered";*)
(*require "$.mlabstractuiwinnt.lfnmisc";*)
(*require "$.mlabstractuiwinnt.lfngeometry";*)
(*require "$.mlabstractuiwinnt.lfnmiscnotify";*)
(*require "$.mlabstractuiwinnt.lfnpointernotify";*)
(*require "$.mlabstractuiwinnt.lfnsimpletext";*)
(*require "protocolaccessorcontr";*)

signature UI_CONTROL_PALETTE_CONTR =
  sig
    val createController : DGNodes.t ->
                           ProtocolAccessorContr.t ->
                           {controlPalette : DGNodes.t} ->
                           {isTCConstructed : DGNodes.t -> bool,
                            createNamedControl : string -> DGNodes.t}
  end

functor FUIControlPaletteContr() :> UI_CONTROL_PALETTE_CONTR =
  struct
    open DGShorthand

    fun createController masterNode
                         {access, clear,
                          getAccessed, checkBinding,
                          addBindDesc}
                         {controlPalette} =
          let
            val buttonHeight = 18
            val regs = Reg.registered ()
          
            val masterCh = masterNode +: "Children"

            val winCh = controlPalette +: "Children"
            val _ = LFNGeometry.setSizeY controlPalette (length regs * buttonHeight)

            val consOptRef = ref NONE

            fun addTCConstructed node =
                  DGPNull.add node "TCConstructed" ()

            fun isTCConstructed node =
                  let
                    val n = hd (node +# "TCConstructed")
                  in
                    true
                  end
                    handle _ => false

            (* FIXME: Don't ever copy code... *)
            fun createNamedControl controlName =
                  let
                    fun doCreate () =
                          let
                            val newControl = Reg.createA controlName
                            val _ = addTCConstructed newControl

                            fun onMouseDown _ {button, point} =
                                  access newControl

                            val _ = LFNPointerNotify.addPointerDownNotifier
                                      newControl onMouseDown
                          in
                            newControl
                          end

                    fun doCreateTopLevel () =
                          let
                            val topLevel = Reg.createA controlName
                            val _ = addTCConstructed topLevel
                            val topLevelCh = topLevel +: "Children"

                            fun onMouseDown _ {button, point} =
                                  case !consOptRef of
                                    NONE => access topLevel
                                  | SOME(control) =>
                                      (consOptRef := NONE;
                                       topLevelCh ~+ control;
                                       LFNGeometry.placeAt control point;
                                       LFNMisc.createControl control;
                                       access control)
                            
                            val _ = LFNPointerNotify.addPointerDownNotifier
                                      topLevel onMouseDown
                          in
                            topLevel
                          end
                  in
                    if controlName = "Window" then
                      doCreateTopLevel ()
                    else
                      doCreate ()
                  end

            fun addCreateButton (controlName, cnt) =
                  let
                    val createButton = Reg.createA "Button"
                    val _ = winCh ~+ createButton
                    val _ = LFNSimpleText.setText createButton controlName
                    val _ = LFNGeometry.setPosY createButton (cnt * buttonHeight)

                    fun doCreate _ () =
                          let
                            val newControl = Reg.createA controlName
                            val _ = addTCConstructed newControl

                            fun onMouseDown _ {button, point} =
                                  access newControl

                            val _ = LFNPointerNotify.addPointerDownNotifier
                                      newControl onMouseDown
                          in
                            consOptRef := SOME(newControl)
                          end

                    fun doCreateTopLevel _ () =
                          let
                            val topLevel = Reg.createA controlName
                            val _ = addTCConstructed topLevel
                            val _ = masterCh ~+ topLevel
                            val topLevelCh = topLevel +: "Children"

                            fun onMouseDown _ {button, point} =
                                  case !consOptRef of
                                    NONE => access topLevel
                                  | SOME(control) =>
                                      (consOptRef := NONE;
                                       topLevelCh ~+ control;
                                       LFNGeometry.placeAt control point;
                                       LFNMisc.createControl control;
                                       access control)
                            
                            val _ = LFNPointerNotify.addPointerDownNotifier
                                      topLevel onMouseDown
                          in
                            (LFNMisc.createControl topLevel;
                             access topLevel)
                          end

                    val _ = if controlName = "Window" then
                              LFNMiscNotify.addActionNotifier createButton doCreateTopLevel
                            else
                              LFNMiscNotify.addActionNotifier createButton doCreate
                  in
                    cnt + 1
                  end

            val _ = List.foldl addCreateButton 0 regs
          in
            {isTCConstructed = isTCConstructed,
             createNamedControl = createNamedControl}
          end
  end

structure UIControlPaletteContr = FUIControlPaletteContr()
