(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "abssynsimplesml_sig";*)

functor FAbsSynSimpleSML() : ABS_SYN_SIMPLE_SML =
  struct
    type symbol = string
    type typ = unit

    datatype var =
      SimpleVar of symbol
    and expr =
      VarExpr of var
    | IntExpr of int
    | RealExpr of real
    | StringExpr of string
    | CharExpr of char
    | BoolExpr of bool
    | UnitExpr
    | CallExpr of {func : symbol, args : expr list}
    | OpExpr of {left : expr, oper : symbol, right : expr}
    | OperaExpr of {operas : expr list}
    | RecordExpr of {fields : (symbol * expr) list, typ : typ}
    | LetExpr of {decls : decl list, body : expr list}
    | NotImplExpr
    and decl =
      FuncDecl of {name : symbol, params : field list,
                   result : typ, body : expr}
    | ValDecl of {pattern : pattern, typ : typ option, expr : expr}
    | TypeDecl of {name : symbol, typ : typ}
    | OpenDecl of {structs : symbol list}
    | Structure of {decls : decl list}
    | NotImplDecl
    and pattern =
      WildcardPattern
    | SymbolPattern of symbol
    (*and typ =
      NameType of symbol
    | RecordType of field list *)
    and field =
      Field of {name : symbol, typ : typ}

    type program = decl list
  end

structure AbsSynSimpleSML = FAbsSynSimpleSML()
