(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "sml97key_sig";*)
(*require "$.basis.__string_cvt";*)
(*require "$.parsingtoolkitwinnt.lexmisc";*)
(*require "simplesml97_grm.sml";*)
(*require "$.errorhandlingwinnt.exceptions";*)
(*require "$.basis.__char";*)
(*require "$.basis.__string";*)

functor FSML97Key(Tokens : SML97_TOKENS) : SML97_KEY =
  struct
    type token = (Tokens.svalue, int) Tokens.token

    (* FIXME: Do this right! *)
    fun vfl (value, pos, rg) = ((value, pos, pos + rg))
  
    fun checkKeyword (yytext, yypos) =
          let
            (* FIXME: Do this right! *)
            fun fl (pos, rg) = (pos, pos + rg)
          in
            case yytext of
            (* Symbols which are also valid symbolic names *)
              ":" => (Tokens.COLON( fl (yypos, 1) ))
            | "=" => (Tokens.EQUAL( fl (yypos, 1) ))
            | ":>" => (Tokens.OPAQUECONSTR( fl (yypos, 2) ))
            | "|" => (Tokens.PIPE( fl (yypos, 1) ))
            | "=>" => (Tokens.EQUALGREATER( fl (yypos, 2) ))
            | "->" => (Tokens.DASHGREATER( fl (yypos, 2) ))
            | "#" => (Tokens.HASH( fl (yypos, 1) ))
            | "*" => (Tokens.ASTERISK( fl (yypos, 1) ))

            (* Keywords *)
            | "if" => (Tokens.IF( fl (yypos, 2) ))
            | "then" => (Tokens.THEN( fl (yypos, 4) ))
            | "else" => (Tokens.ELSE( fl (yypos, 4) ))
            | "and" => (Tokens.AND( fl (yypos, 3) ))
            | "op" => (Tokens.OP( fl (yypos, 2) ))
            | "structure" => (Tokens.STRUCTURE( fl (yypos, 9) ))
            | "signature" => (Tokens.SIGNATURE( fl (yypos, 9) ))
            | "struct" => (Tokens.STRUCT( fl (yypos, 6) ))
            | "functor" => (Tokens.FUNCTOR( fl (yypos, 7) ))
            | "local" => (Tokens.LOCAL( fl (yypos, 5) ))
            | "in" => (Tokens.IN( fl (yypos, 2) ))
            | "end" => (Tokens.END( fl (yypos, 3) ))
            | "orelse" => (Tokens.ORELSE( fl (yypos, 6) ))
            | "andalso" => (Tokens.ANDALSO( fl (yypos, 7) ))
            | "val" => (Tokens.VAL( fl (yypos, 3) ))
            | "type" => (Tokens.TYPE( fl (yypos, 4) ))
            | "eqtype" => (Tokens.EQTYPE( fl (yypos, 6) ))
            | "datatype" => (Tokens.DATATYPE( fl (yypos, 8) ))
            | "abstype" => (Tokens.ABSTYPE( fl (yypos, 7) ))
            | "with" => (Tokens.WITH( fl (yypos, 4) )) 
            | "exception" => (Tokens.EXCEPTION( fl (yypos, 9) ))
            | "open" => (Tokens.OPEN( fl (yypos, 4) ))
            | "withtype" => (Tokens.WITHTYPE( fl (yypos, 8) ))
            | "let" => (Tokens.LET( fl (yypos, 3) ))
            | "sig" => (Tokens.SIG( fl (yypos, 3) ))
            | "where" => (Tokens.WHERE( fl (yypos, 5) ))
            | "sharing" => (Tokens.SHARING( fl (yypos, 7) ))
            | "include" => (Tokens.INCLUDE( fl (yypos, 7) ))
            | "infix" => (Tokens.INFIX( fl (yypos, 5) ))
            | "infixr" => (Tokens.INFIXR( fl (yypos, 6) ))
            | "nonfix" => (Tokens.NONFIX( fl (yypos, 6) ))
            | "while" => (Tokens.WHILE( fl (yypos, 5) ))
            | "do" => (Tokens.DO( fl (yypos, 2) ))
            | "handle" => (Tokens.HANDLE( fl (yypos, 6) ))
            | "raise" => (Tokens.RAISE( fl (yypos, 5) ))
            | "fn" => (Tokens.FN( fl (yypos, 2) ))
            | "case" => (Tokens.CASE( fl (yypos, 4) ))
            | "as" => (Tokens.AS( fl (yypos, 2) ))
            | "rec" => (Tokens.REC( fl (yypos, 3) ))
            | "fun" => (Tokens.FUN( fl (yypos, 3)))

            (* Otherwise it's an identifier *)
            | _ => (Tokens.IDENT( vfl (yytext, yypos, size yytext) ))
          end

    fun getStringOrChar (yytext, yypos) =
          let
            fun errStr beforeChar errorString =
                  Excn.eRaiseBug
                    ("There's a bug in the SML'97 lexer or " ^
                     "in the SML97Key.getStringOrChar function! " ^
                     "Unknown escape char after " ^
                     (Char.toString beforeChar) ^ ": " ^
                     errorString)
          
            fun err beforeChar errorChar =
                  errStr beforeChar (Char.toString errorChar)

            fun eatBreak getc strm =
                  case getc strm of
                    NONE => errStr #" " "<NO MORE CHARACTERS IN BROKEN STRING>"
                  | SOME(a, strma) =>
                      (case a of
                         #" " => eatBreak getc strma
                       | #"\t" => eatBreak getc strma
                       | #"\n" => eatBreak getc strma
                       | #"\f" => eatBreak getc strma
                       | #"\\" => SOME("", strma)
                       | _ => err #" " a)

            fun escapeBackslash getc strm =
                  case getc strm of
                    NONE => errStr #"\\" "<NO MORE CHARACTERS>"
                  | SOME(a, strma) =>
                      (case a of
                         #"\\" => SOME("\\", strma)
                       | #"\"" => SOME("\"", strma)
                       | #"a" => SOME("\a", strma)
                       | #"b" => SOME("\b", strma)
                       | #"f" => SOME("\f", strma)
                       | #"n" => SOME("\n", strma)
                       | #"r" => SOME("\r", strma)
                       | #"t" => SOME("\t", strma)
                       | #"v" => SOME("\v", strma)
                       | #"^" => Excn.eRaiseNotImplemented
                                   ("In SML97Key.getStringOrChar: " ^
                                    "Escape codes beginning with \\^ not implemented!")
                       | #" " => eatBreak getc strma
                       | #"\t" => eatBreak getc strma
                       | #"\n" => eatBreak getc strma
                       | #"\f" => eatBreak getc strma
                       | _ => err #"\\" a)
          
            fun escapeChar getc strm =
                  case getc strm of
                    NONE => NONE
                  | SOME(a, strma) =>
                      (case a of
                         #"\\" =>
                           (case getc strma of
                              NONE => errStr #"\\" "<NO MORE CHARACTERS>"
                            | SOME(b, strmb) =>
                                (case b of
                                   #"\\" => SOME("\\", strmb)
                                 | #"n" => SOME("\n", strmb)
                                 | #"\"" => SOME("\"", strmb)
                                 | _ => err a b))
                       | _ => SOME(Char.toString a, strma))

            fun escapeIt concatWith getc strm =
                  case escapeChar getc strm of
                    NONE => SOME(concatWith, strm)
                  | SOME(str, rest) =>
                      escapeIt (concatWith ^ str) getc rest
          
            fun convertIt inString =
                  case StringCvt.scanString (escapeIt "") inString of
                    NONE => Excn.eRaiseBug ("Bug in SML97Key.getStringOrChar function!")
                  | SOME(outString) => outString
          in
            (* Check if it's a char or a string, and trim off the quotes *)
            if String.isPrefix "#" yytext then
              case Char.fromString
                     (convertIt
                        (LexMisc.trimlr 2 1 yytext)) of
                NONE =>
                  Excn.eRaiseGeneric
                    ("PARSE_ERROR",
                     "Illegal character constant! (FIXME: No position reported...)")
              | SOME(c) =>
                  Tokens.LITCHAR ( vfl(c, yypos, size yytext))
            else
              Tokens.LITSTRING
                (vfl (convertIt
                        (LexMisc.trimlr 1 1 yytext),
                      yypos, size yytext))
          end
  end
