(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.parsingtoolkitwinnt.lexerrormsg";*)
(*require "abssynsimplesml";*)
(*require "$.basis.__text_io";*)
(*require "$.errorhandlingwinnt.exceptions";*)
(*require "$.commonfunctions.util";*)
(*require "simplesml97_grm";*)
(*require "_simplesml97_grm";*)
(*require "sml97_lex";*)
(*require "$.mlyacc.base";*)
(*require "$.mlyacc.parser2";*)
(*require "$.mlyacc.join";*)
(*require "$.mlyacc.lrtable";*)
(*require "$.mlyacc.stream";*)


functor FSimpleSML97Parser () (* FIXME: Make a signature *) =
  struct
    structure SML97LrVals = FSML97LrValsFun(
                              structure Token = LrParser.Token
                              structure Abs = AbsSynSimpleSML)
    structure SML97Lex = FSML97LexFun(
                           structure Tokens = SML97LrVals.Tokens)
    structure SMLParser = Join(structure ParserData = SML97LrVals.ParserData
			structure Lex = SML97Lex
			structure LrParser = LrParser)

    fun scanFile filename =
      let
        val _ = ErrorMsg.reset()
        val inFile = TextIO.openIn filename
      in
        Util.finalize
          (fn _ =>
             let
               fun get _ = TextIO.input inFile
               fun parseError(s,p1,p2) = ErrorMsg.error p1 s
             in
               SML97Lex.makeLexer get
             end)
          (fn _ => TextIO.closeIn inFile)
      end
        handle LrParser.ParseError =>
          Excn.eRaiseGeneric ("SimpleSML97Parse",
                              "Error occurred during scanning!")
    
    fun parseFile filename =
      let
        val _ = ErrorMsg.reset()
        val inFile = TextIO.openIn filename
      in
        Util.finalize
          (fn _ =>
             let
               fun get _ = TextIO.input inFile
               fun parseError(s,p1,p2) = ErrorMsg.error p1 s
               val lexer = LrParser.Stream.streamify
                     (SML97Lex.makeLexer get)
               val (abssyn, _) = SMLParser.parse(15, lexer, parseError, ())
             in
               abssyn : AbsSynSimpleSML.program
             end)
          (fn _ => TextIO.closeIn inFile)
      end
        handle LrParser.ParseError =>
          Excn.eRaiseGeneric ("SimpleSML97Parse",
                              "Error occurred during parsing!")
  end

structure SimpleSML97Parser = FSimpleSML97Parser()
