(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gtwinnt.dgnodes";*)
(*require "$.collectionswinnt.simplesets";*)
(*require "$.gtwinnt.dgshorthand";*)
(*require "$.basis.__list";*)
(*require "$.basis.__int";*)
(*require "$.basis.__text_io";*)
(*require "$.commonfunctionswinnt.util";*)
(*require "protocolmanip";*)
(*require "$.basis.__os_path";*)
(*require "$.system.__os";*)

functor FSimpleSMLPrettyPrint() =
  struct
    (* FIXME: Also in ProtocolAccessorContr... *)
    val excludeList = ["Create", "Children",
                       "TCConstructed", "BindingName"]

    fun prettyPrintProtocolNode outF space
                                getBindingName protocolNode
                                fromBinding =
          let
            fun outLF () = (outF "\n"; outF space)
            val typeName = DGNodes.typeName protocolNode
          in
            case ProtocolManip.getProtocol
                    (DGNodes.protocolName protocolNode) of
              NONE => ()
            | SOME({readAsString, writeAsString, saveSML}) =>
                let
                  val bindingName = "tmpProtNode"
                  val protBindingName = getBindingName protocolNode "protocol"
                in
                  (outF ("val " ^ bindingName ^ " = ");
                   outF fromBinding;
                   outF " +: \""; outF typeName;
                   outF "\""; outLF ();
                   saveSML (outF, outLF) (protocolNode, bindingName, protBindingName))
                end
          end

    fun prettyPrintNode outF space getBindingName
                        isTCConstructed node fromBinding =
          let
            val nodes = DGNodes.connected node
            fun outLF () = (outF "\n"; outF space)

            fun handleNode node =
                  let
                    val typeName = DGNodes.typeName node
                  in
                    if SimpleSets.contains typeName excludeList then
                      ()
                    else
                      prettyPrintProtocolNode outF space getBindingName
                                              node fromBinding
                  end

            fun printChildren () =
                  let
                    open DGShorthand
                    val nodeCh = node +: "Children"
                    val childrenBindName = getBindingName nodeCh "children"
                    val childNodes = DGNodes.connected nodeCh

                    val _ = if length childNodes > 0 then
                              (outLF ();
                               outF ("val " ^ childrenBindName ^ " = ");
                               outF fromBinding;
                               outF " +: \"Children\"";
                               outLF ())
                            else ()

                    fun printChild childNode =
                          if isTCConstructed childNode then
                            let
                              val typeName = DGNodes.typeName childNode
                              val childBindName = getBindingName childNode "control"
                              val _ = outLF ()
                              val _ = outF ("val " ^ childBindName ^
                                            " = Reg.createA \"" ^
                                            typeName ^ "\"")
                              val _ = outLF ()
                              val _ = outF ("val _ = " ^ childrenBindName ^ " ~+ " ^ childBindName)
                              val _ = outLF ()
                            in
                              prettyPrintNode outF space getBindingName
                                              isTCConstructed childNode
                                              childBindName
                            end
                          else ()
                  in
                    List.app printChild childNodes
                  end
          in
            (List.app handleNode nodes;
             printChildren ())
          end
  
    fun prettyPrint outF space checkBinding isTCConstructed windowNode =
          let
            val bindings = ref nil
            val uniquenessCounter = ref 0

            fun getStrCnt () =
                  let
                    val c = !uniquenessCounter
                  in
                    (uniquenessCounter := c + 1;
                     Int.toString c)
                  end

            fun getBindingName node defaultBaseName =
                  let
                    val (_, boundName) = checkBinding node
                  in
                    if boundName = "" then
                      defaultBaseName ^ (getStrCnt ())
                    else
                      (bindings := boundName :: (!bindings);
                       boundName)
                  end
          
            val _ = outF space
            val winBinding = getBindingName windowNode "window"
            val _ = outF "val masterCh = masterNode +: \"Children\"\n"
            val _ = outF space
            val _ = outF ("val " ^ winBinding ^ " = Reg.createA \"Window\"\n")
            val _ = outF space
            val _ = outF ("val _ = masterCh ~+ " ^ winBinding ^ "\n")
            val _ = outF space

            val _ = prettyPrintNode outF space getBindingName
                                    isTCConstructed windowNode winBinding
          in
            !bindings
          end
  
    fun saveStart fileName outF =
          let
            val _ = outF "functor F"
            val _ = outF (OS.Path.base fileName)
            val _ = outF "() =\n"
            val _ = outF "  struct\n"

            val _ = outF "    fun createGraph "
            val _ = outF "masterNode =\n"
            val _ = outF "          let\n"
            val _ = outF "            open DGShorthand\n"
            val _ = outF "\n"
          in
            ()
          end

    fun saveGraph checkBinding isTCConstructed windowNode outF =
          let
            fun addBinding (name, genComma) =
                  (if genComma then
                     (outF ",\n"; outF "             ")
                   else ();
                   outF name; outF " = ";
                   outF name; true)
          
            val bindings = prettyPrint outF
                             "            "
                             checkBinding isTCConstructed windowNode

            val _ = outF "\n          in\n"
            val _ = outF "            {"
            val _ = List.foldl addBinding false bindings
            val _ = outF "}\n"
            val _ = outF "          end\n"
          in
            ()
          end

    fun saveEnd fileName outF =
          let
            val _ = outF "  end\n\n"
            val _ = outF "structure "
            val _ = outF (OS.Path.base fileName)
            val _ = outF " = F"
            val _ = outF (OS.Path.base fileName)
            val _ = outF "()\n"
          in
            ()
          end
  
    fun saveOutF checkBinding isTCConstructed windowNode fileName outF =
          let
            val _ = saveStart fileName outF
            val _ = saveGraph checkBinding isTCConstructed windowNode outF
            val _ = saveEnd fileName outF
          in
            ()
          end
  
    fun saveFile checkBinding isTCConstructed windowNode fileName =
          let
            val outStr = TextIO.openOut fileName

            fun outF value =
                  TextIO.output (outStr, value)
          in
            Util.finalize
               (fn () => (saveOutF checkBinding isTCConstructed
                            windowNode fileName outF;
                          TextIO.flushOut outStr))
               (fn () => TextIO.closeOut outStr)
          end
  end

structure SimpleSMLPrettyPrint =
  FSimpleSMLPrettyPrint()
