(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "protocolmanip";*)
(*require "$.basis.__list";*)
(*require "$.basis.__string";*)
(*require "abssynsimplesml_sig";*)
(*require "abssynsimplesml";*)
(*require "$.mlabstractui.uiwindowstoportable";*)
(*require "$.gtwinnt.dgnodes";*)
(*require "$.gtwinnt.dgshorthand";*)
(*require "$.dgpcommon";*)
(*require "$.mlabstractui.pnotifiers";*)
(*require "$.mlabstractui.uiprotocols";*)
(*require "$.collectionswinnt.dictionary";*)
(*require "$.collectionswinnt.simplesets";*)
(*require "$.errorhandlingwinnt.exceptions";*)

functor FSimpleSMLUIEval (Abs : ABS_SYN_SIMPLE_SML) =
  struct
    open DGShorthand

    datatype assoc =
      LeftAssoc
    | RightAssoc
    | NonAssoc
  
    val operators = ref (StringDict.empty :
                           {assoc : assoc} StringDict.t)

    fun addOperator name assoc =
          operators :=
            StringDict.insert
              (!operators)
              name
              {assoc = assoc}

    fun isOperator (Abs.VarExpr(Abs.SimpleVar(name))) =
          (case StringDict.lookup (!operators) name of
             NONE => false
           | SOME(_) => true)
      | isOperator _ = false

    fun functionArgs (Abs.VarExpr(Abs.SimpleVar(name))) =
          ProtocolManip.functionArgs name
      | functionArgs _ = 0

    fun getSymbol (Abs.VarExpr(Abs.SimpleVar(name))) =
          name
      | getSymbol _ =
          Excn.eRaiseBug("Bug in SimpleSMLUIEval")

    val _ = addOperator "+:" LeftAssoc
    val _ = addOperator "~+" LeftAssoc

    fun arrangeCalls nil = nil
      | arrangeCalls (e::rest) =
          let
            val args = functionArgs e
          in
            if args = 0 then
              e::arrangeCalls rest
            else
              (Abs.CallExpr{func = getSymbol(e),
                            args = List.take(rest, args)})
                 ::arrangeCalls (List.drop(rest, args))
          end

    (* FIXME: Only left associtive operators are supported
              and this does problably not even work when mixing
              with function application.  *)
    fun arrangeOperators NONE nil = nil
      | arrangeOperators (SOME(prev)) nil = [prev]
      | arrangeOperators NONE [opera] = [opera]
      | arrangeOperators (SOME(prev)) [opera] = [prev, opera]
      | arrangeOperators NONE (opera1::opera2::rest) =
          if isOperator opera1 then
            Excn.eRaiseGeneric("SMLUIEval", "No left operand for operator " ^
                                            (getSymbol opera1))
          else
            arrangeOperators (SOME(opera1)) (opera2::rest)
      | arrangeOperators (SOME(prev)) (opera1::opera2::rest) =
          if isOperator opera1 then
            (* Here we assume left associativity *)
            arrangeOperators
              (SOME (Abs.OpExpr{left = prev,
                                oper = getSymbol opera1,
                                right = opera2}))
              rest
          else
            prev :: arrangeOperators (SOME(opera1)) (opera2::rest)

    fun preArrange operas =
          hd (arrangeOperators NONE (arrangeCalls operas))

    fun getBindings [Abs.OperaExpr{operas =
                       [Abs.RecordExpr {fields, typ}]}] =
          map #1 fields
      | getBindings _ =
          Excn.eRaiseGeneric("SMLUIEval",
                             ("Expecting a record-expression as " ^
                              "return value in the let-expression!"))

    datatype SMLBindings =
      IntProt of PInt.t
    | BoolProt of PBool.t
    | StringProt of PString.t
    | StringsProt of PStrings.t
    | NotifyProt of PNotify.t
    | PointerButtonNotifyProt of PPointerButtonNotify.t
    | ItemSelectionsProt of PItemSelections.t
    | GraphNode of DGNodes.t
    | Unit

    (* Like getSymbol above but without exception *)
    fun getVarName (Abs.VarExpr(Abs.SimpleVar(n))) =
          n
      | getVarName _ = ""

    fun getLitString (Abs.StringExpr(s)) =
          s
      | getLitString _ =
          Excn.eRaiseGeneric("SMLUIEval",
                             ("String constant expected!"))

    fun getLitInt (Abs.IntExpr(i)) =
          i
      | getLitInt _ =
          Excn.eRaiseGeneric("SMLUIEval",
                             ("Integer constant expected!"))

    fun getLitBool expr =
          case getSymbol expr of
            "true" => true
          | "false" => false
          | _ =>
              Excn.eRaiseGeneric("SMLUIEval",
                                 ("Boolean constant expected!"))

    fun getNamedNode n nil =
          Excn.eRaiseGeneric("SMLUIEval",
                             ("Non-existant binding: " ^ n))
      | getNamedNode n ({decl = GraphNode(aNode), name}::rest) =
          if name = n then
            aNode
          else getNamedNode n rest
      | getNamedNode n (_::rest) =
          getNamedNode n rest

    fun getNamedPInt n nil =
          Excn.eRaiseGeneric("SMLUIEval",
                             ("Non-existant binding: " ^ n))
      | getNamedPInt n ({decl = IntProt(aProt), name}::rest) =
          if name = n then
            aProt
          else getNamedPInt n rest
      | getNamedPInt n (_::rest) =
          getNamedPInt n rest

    fun getNamedPBool n nil =
          Excn.eRaiseGeneric("SMLUIEval",
                             ("Non-existant binding: " ^ n))
      | getNamedPBool n ({decl = BoolProt(aProt), name}::rest) =
          if name = n then
            aProt
          else getNamedPBool n rest
      | getNamedPBool n (_::rest) =
          getNamedPBool n rest

    fun getNamedPString n nil =
          Excn.eRaiseGeneric("SMLUIEval",
                             ("Non-existant binding: " ^ n))
      | getNamedPString n ({decl = StringProt(aProt), name}::rest) =
          if name = n then
            aProt
          else getNamedPString n rest
      | getNamedPString n (_::rest) =
          getNamedPString n rest

    fun evalExpr createNamedControl bindings (Abs.OperaExpr{operas}) =
          evalExpr createNamedControl bindings (preArrange operas)
      | evalExpr createNamedControl bindings (Abs.CallExpr{func, args}) =
          (case func of
             "Reg.createA" =>
                GraphNode(createNamedControl (getLitString(hd args)))
           | "PInt.write" =>
                (PInt.write (getNamedPInt (getVarName (hd args)) bindings)
                            (getLitInt(hd (tl args))); Unit)
           | "DGPInt.p" =>
                IntProt (DGPInt.p (getNamedNode (getVarName (hd args)) bindings))
           | "PBool.write" =>
                (PBool.write (getNamedPBool (getVarName (hd args)) bindings)
                            (getLitBool(hd (tl args))); Unit)
           | "DGPBool.p" =>
                BoolProt (DGPBool.p (getNamedNode (getVarName (hd args)) bindings))
           | "PString.write" =>
                (PString.write (getNamedPString (getVarName (hd args)) bindings)
                            (getLitString(hd (tl args))); Unit)
           | "DGPString.p" =>
                StringProt (DGPString.p (getNamedNode (getVarName (hd args)) bindings))
           | "DGPStrings.p" =>
                StringsProt (DGPStrings.p (getNamedNode (getVarName (hd args)) bindings))
           | "DGPNotify.p" =>
                NotifyProt (DGPNotify.p (getNamedNode (getVarName (hd args)) bindings))
           | "DGPPointerButtonNotify.p" =>
                PointerButtonNotifyProt (DGPPointerButtonNotify.p (getNamedNode (getVarName (hd args)) bindings))
           | "DGPItemSelections.p" =>
                ItemSelectionsProt (DGPItemSelections.p (getNamedNode (getVarName (hd args)) bindings))
           | _ =>
               Excn.eRaiseGeneric("SMLUIEval",
                 ("Unsupported function: " ^ func)))
      | evalExpr _ bindings (Abs.OpExpr{left, oper, right}) =
          (case oper of
             "+:" => GraphNode((getNamedNode (getVarName left) bindings)
                                  +: (getLitString right))
           | "~+" => ((getNamedNode (getVarName left) bindings)
                         ~+ (getNamedNode (getVarName right) bindings); Unit)
           | _ =>
               Excn.eRaiseGeneric("SMLUIEval",
                 ("Unsupported operator: " ^ oper)))
      | evalExpr _ _ _ =
          Excn.eRaiseGeneric("SMLUIEval",
            ("Operator or function call expression expected!"))

    (* This function is necessary because in the file,
       the _protocols_ are bound - not the protocol nodes.
       It adds the binding description to the protocol node
       that the protocol binding came from... *)
    fun checkAddBindDesc addBindDesc newBindings name
                           (Abs.OperaExpr{operas}) =
          checkAddBindDesc addBindDesc newBindings name
            (preArrange operas)
      | checkAddBindDesc addBindDesc newBindings name
                         (Abs.CallExpr{func, args}) =
          let
            val symSize = String.size func
            val argName = getVarName (hd args)
          in
            if String.sub (func, symSize - 2) = #"." andalso
               String.sub (func, symSize - 1) = #"p" then
              addBindDesc (getNamedNode argName newBindings) name
            else ()
          end
      | checkAddBindDesc _ _ _ _ = ()

    fun evalDecl addBindDesc createNamedControl
                   publicBindings bindings
                   (Abs.ValDecl{pattern, typ, expr}) =
          (case pattern of
             Abs.WildcardPattern =>
               (evalExpr createNamedControl bindings expr; bindings)
           | Abs.SymbolPattern(s) =>
               let
                 val newDecl = evalExpr createNamedControl bindings expr
                 val newBindings =
                       {decl = newDecl, name = s} :: bindings
        
                 val _ = if SimpleSets.contains s publicBindings then
                           case newDecl of
                             GraphNode(aNode) =>
                               addBindDesc aNode s
                           | _ =>
                               checkAddBindDesc
                                 addBindDesc newBindings s expr
                         else ()
               in
                 newBindings
               end)        
      | evalDecl _ _ _ bindings (Abs.OpenDecl _) =
          bindings
      | evalDecl _ _ _ _ _ =
          Excn.eRaiseGeneric("SMLUIEval",
                             ("Expecting only val and open declarations " ^
                              "in the let-expression!"))

    fun evalDecls _ _ _ _ nil = ()
      | evalDecls addBindDesc createNamedControl
                    publicBindings bindings (decl::decls) =
          let
            val newBindings =
                  evalDecl addBindDesc createNamedControl
                           publicBindings bindings decl
          in
            evalDecls addBindDesc createNamedControl
                      publicBindings newBindings decls
          end

    fun extractLetExpr ((Abs.Structure{decls}::_) : Abs.program) =
          (case decls of
             (Abs.FuncDecl{name, params, result, body}::_) =>
                (case body of
                   (Abs.OperaExpr{operas}) =>
                      (case operas of
                         ((Abs.LetExpr{decls, body})::_) =>
                            (decls, body)
                       | _ => Excn.eRaiseGeneric("SMLUIEval",
                                "The expected let-expression is not found!"))
                 | _ => Excn.eRaiseGeneric("SMLUIEval",
                          "The expected operators/operands expression is not found!"))
           | _ => Excn.eRaiseGeneric("SMLUIEval",
                    "The expected function declaration is not found!"))
      | extractLetExpr _ =
          Excn.eRaiseGeneric("SMLUIEval",
            "The expected main-structure is not found!")
  
    fun smlAbsSynToUI masterNode addBindDesc createNamedControl prog =
          let
            val (decls, body) = extractLetExpr prog
            
            val publicBindings = getBindings body
            val bindings = [{decl = GraphNode(masterNode), name = "masterNode"}]
            val _ = evalDecls addBindDesc createNamedControl
                      publicBindings bindings decls
          in
            (* FIXME: Only create controls for loaded window... *)
            uiCore.createGUI masterNode
          end
  end

structure SimpleSMLUIEval = FSimpleSMLUIEval(AbsSynSimpleSML)
