(*
   This file is part of The Construct - a graphical user interface
   builder for ML::AbstractUI written in Standard ML.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "simplesml97_grm";  *)
(*require "sml97key";       *)
(*require "$.parsingtkwinnt.lexerrormsg";*)
(*require "$.parsingtkwinnt.lexmisc";*)

structure Key = FSML97Key(Tokens)

type pos = int
type svalue = Tokens.svalue
type ('a, 'b) token = ('a, 'b) Tokens.token
type lexresult = (svalue, pos) token

val lineNum = ErrorMsg.lineNum
val linePos = ErrorMsg.linePos
val error = ErrorMsg.error
val commentNest = ErrorMsg.commentNest

val decStringToInt  = LexMisc.decStringToInt

val decStringToReal = LexMisc.decStringToReal

(* FIXME: Do this right! *)
fun fl (pos, rg) = (pos, pos + rg);
fun vfl (value, pos, rg) = ((value, pos, pos + rg));

fun eof() = let val pos = hd(!linePos) in Tokens.EOF( fl (pos, 0)) end

%%
%header (functor FSML97LexFun(
           structure Tokens : SML97_TOKENS));
%full

ws=[\ \t];
digit=[0-9];
letter=[A-Za-z];
symbol=[!%&$#+-/:<=>?@\~`^|*];
hatescape=[A-Z@\\_^]|"["|"]";
stdescape=[abfnrtv];
strws=[\ \t\n\f];
alphanumident={letter}(({letter}|{digit}|"_"|"'")*); 
identifier={alphanumident}|({symbol}+);

%s MLCOMMENT;

%%

\n                    => (lineNum := !lineNum+1;
                          linePos := yypos :: !linePos;
                          continue());
<INITIAL>{ws}+        => (continue());
<INITIAL>"(*"         => (YYBEGIN MLCOMMENT; continue ());
<INITIAL>"~"{digit}+  => (Tokens.LITNEGINT(
                            vfl (decStringToInt (yytext, yypos),
                                 yypos, size yytext)));
<INITIAL>{digit}+     => (Tokens.LITPOSINT(
                            vfl (decStringToInt (yytext, yypos),
                                 yypos, size yytext)));
<INITIAL>"["          => (Tokens.LBRACKET( fl (yypos, 1) ));
<INITIAL>"]"          => (Tokens.RBRACKET( fl (yypos, 1) ));
<INITIAL>"{"          => (Tokens.LCBRACE( fl (yypos, 1) ));
<INITIAL>"}"          => (Tokens.RCBRACE( fl (yypos, 1) ));
<INITIAL>"("          => (Tokens.LPAREN( fl (yypos, 1) ));
<INITIAL>")"          => (Tokens.RPAREN( fl (yypos, 1) ));
<INITIAL>","          => (Tokens.COMMA( fl (yypos, 1) ));
<INITIAL>";"          => (Tokens.SEMICOLON( fl (yypos, 1) ));
<INITIAL>"..."        => (Tokens.DOTDOTDOT( fl (yypos, 3) ));
<INITIAL>"_"          => (Tokens.UNDERSCORE( fl (yypos, 1) ));
<INITIAL>{identifier} => (Key.checkKeyword (yytext, yypos));
<INITIAL>{identifier}("."{identifier})+
                      => (Tokens.COMPIDENT(
                            vfl (yytext, yypos, size yytext)
                            ));
<INITIAL>"'""'"?"_"?{alphanumident}
                      => (Tokens.TYPEVAR(
                            vfl (yytext, yypos, size yytext)
                            ));
<INITIAL>"~"?{digit}+("."{digit}+)("E""~"?{digit}+)?
                      => (Tokens.LITREAL(
                            vfl (decStringToReal (yytext, yypos),
                                 yypos, size yytext)));
<INITIAL>"~"?{digit}+("E""~"?{digit}+) =>
                         (Tokens.LITREAL(
                            vfl (decStringToReal (yytext, yypos),
                                 yypos, size yytext)));
<INITIAL>"#"?\"([^\\\"\n]|(\\({stdescape}|
                              ("^"{hatescape})|
                              ({digit}{3})|
                              (\") | (\\) |
                              ({strws}+\\)
                             )))*\" =>
                         (Key.getStringOrChar(yytext, yypos));

<MLCOMMENT>"(*"       => (commentNest := (!commentNest) + 1;
                          continue ());
<MLCOMMENT>"*)"       => (if !commentNest <= 0 then
                            (YYBEGIN INITIAL)
                          else
                            (commentNest := (!commentNest) - 1);
                          continue());
<MLCOMMENT>.          => (continue());

.       => (error yypos ("illegal character " ^ yytext); continue());
